@@ -29,14 +29,9 @@ class Local:
         else: return RepInfo(req.json())
 
     def claim_video_rep(self, chatId: str):
-        info = self.get_video_rep_info(chatId)
-        reputation = info.json["reputation"]
-
-        if int(reputation) < 1: return CheckExceptions(f"reputation should be more than 1 (Your rep Now {int(reputation)}")
-
         req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation"), headers=self.headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
-        else: return Rep(req.json())
+        else: return RepInfo(req.json())
 
     def join_chat(self, chatId: str = None):
         req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}"), headers=self.headers, proxies=self.proxies)
@@ -135,7 +130,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return ThreadList(req.json()["threadList"]).ThreadList
 
-    def send_message(self, chatId: str, message: str = None, messageType: int = 0, file: BinaryIO = None, fileType: str = None, replyTo: str = None, mentionUserIds: list = None, stickerId: str = None, embedId: str = None, embedType: int = None, embedLink: str = None, embedTitle: str = None, embedContent: str = None, embedImage: BinaryIO = None):
+    def send_message(self, chatId: str, message: str = None, messageType: int = 0, file: BinaryIO = None, fileType: str = None, replyTo: str = None, mentionUserIds: list = None, stickerId: str = None, snippetLink: str = None, ytVideo: str = None, snippetImage: BinaryIO = None, embedId: str = None, embedType: int = None, embedLink: str = None, embedTitle: str = None, embedContent: str = None, embedImage: BinaryIO = None):
         if message is not None and file is None: message = message.replace("[@", "‎‏").replace("@]", "‬‭")
 
         mentions = []
@@ -156,6 +151,7 @@ class Local:
                 "mediaList": embedImage
             },
             "extensions": {"mentionedArray": mentions},
+            "clientRefId": int(timestamp() / 10 % 100000000),
             "timestamp": int(timestamp() * 1000)
         }
 
@@ -166,8 +162,23 @@ class Local:
             data["stickerId"] = stickerId
             data["type"] = 3
 
+        if snippetLink and snippetImage:
+            data["attachedObject"] = None
+            data["extensions"]["linkSnippetList"] = [{
+                "link": snippetLink,
+                "mediaType": 100,
+                "mediaUploadValue": base64.b64encode(snippetImage.read()).decode(),
+                "mediaUploadValueContentType": "image/png"
+            }]
+
+        if ytVideo:
+            data["content"] = None
+            data["mediaType"] = 103
+            data["mediaValue"] = ytVideo
+
         if file:
             data["content"] = None
+
             if fileType == "audio":
                 data["type"] = 2
                 data["mediaType"] = 110
@@ -175,17 +186,18 @@ class Local:
             elif fileType == "image":
                 data["mediaType"] = 100
                 data["mediaUploadValueContentType"] = "image/jpg"
-                data["mediaUhqEnabled"] = True
+                data["mediaUhqEnabled"] = False
 
             elif fileType == "gif":
                 data["mediaType"] = 100
                 data["mediaUploadValueContentType"] = "image/gif"
-                data["mediaUhqEnabled"] = True
+                data["mediaUhqEnabled"] = False
 
-            else:
-                raise TypeError("")
+            else: raise TypeError("")
 
             data["mediaUploadValue"] = base64.b64encode(file.read()).decode()
+            data["attachedObject"] = None
+            data["extensions"] = None
 
         data = json.dumps(data)
         req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/message"), headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data)
@@ -223,7 +235,7 @@ class Local:
         data = {"timestamp": int(timestamp() * 1000)}
 
         if isinstance(userId, str): url = api(f"/x{self.comId}/s/user-profile/{userId}/member")
-        if isinstance(userId, list):
+        elif isinstance(userId, list):
             url = api(f"/x{self.comId}/s/user-profile/{self.uid}/joined")
             data = json.dumps({"targetUidList": userId, "timestamp": int(timestamp() * 1000)})
         else: raise TypeError("Please put str or list of userId")
@@ -234,7 +246,7 @@ class Local:
 
     def start_chat(self, userId: [str, list], title: str = None, message: str = None, content: str = None, type: int = 0):
         if isinstance(userId, list): userIds = userId
-        if isinstance(userId, str): userIds = [userId]
+        elif isinstance(userId, str): userIds = [userId]
         else: raise TypeError("Please put a str or list of userId")
 
         data = json.dumps({
@@ -304,60 +316,62 @@ class Local:
         res.append(Json(req.json()))
         return res
 
-    def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None):
+    def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None, coHosts: [str, list] = None):
         res = []
 
-        if doNotDisturb is not None:
+        if doNotDisturb:
             if doNotDisturb: opt = 2
-            if not doNotDisturb: opt = 1
-            else: raise TypeError("Do not disturb should be True or False")
+            else: opt = 1
 
-            data = json.dumps({"alertOption": opt})
-            req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/alert"), data=data, headers=self.headers, proxies=self.proxies)
+            data = json.dumps({"alertOption": opt, "timestamp": int(timestamp() * 1000)})
+            req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/alert"), data=data, headers=headers.Headers(data=data).headers, proxies=self.proxies)
             if req.status_code != 200: return CheckExceptions(req.json())
             res.append(Json(req.json()))
 
-        if viewOnly is not None:
+        if viewOnly:
             if viewOnly: viewOnly = "enable"
-            if not viewOnly: viewOnly = "disable"
-            else: raise TypeError("viewOnly should be True or False")
+            else: viewOnly = "disable"
 
             req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/view-only/{viewOnly}"), headers=self.headers, proxies=self.proxies)
             if req.status_code != 200: return CheckExceptions(req.json())
             res.append(Json(req.json()))
 
-        if canInvite is not None:
+        if canInvite:
             if canInvite: canInvite = "enable"
-            if not canInvite: canInvite = "disable"
-            else: raise TypeError("can invite should be True or False")
+            else: canInvite = "disable"
 
             req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/members-can-invite/{canInvite}"), headers=self.headers, proxies=self.proxies)
             if req.status_code != 200: return CheckExceptions(req.json())
             res.append(Json(req.json()))
 
-        if canTip is not None:
+        if canTip:
             if canTip: canTip = "enable"
-            if not canTip: canTip = "disable"
-            else: raise TypeError("can tip should be True or False")
+            else: canTip = "disable"
 
             req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/tipping-perm-status/{canTip}"), headers=self.headers, proxies=self.proxies)
             if req.status_code != 200: return CheckExceptions(req.json())
             res.append(Json(req.json()))
 
-        if pin is not None:
+        if pin:
             if pin: pin = "pin"
-            if not pin: pin = "unpin"
-            else: raise TypeError("pin should be True or False")
+            else: pin = "unpin"
 
             req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/{pin}"), headers=self.headers, proxies=self.proxies)
             if req.status_code != 200: return CheckExceptions(req.json())
             res.append(Json(req.json()))
 
+        if coHosts:
+            data = json.dumps({"uidList": coHosts, "timestamp": int(timestamp() * 1000)})
+            req = requests.post(api(f"{self.comId}/s/chat/thread/{chatId}/co-host"), data=data, headers=headers.Headers(data=data).headers)
+            if req.status_code != 200: return CheckExceptions(req.json())
+            res.append(Json(req.json()))
+
         return res
 
-    def vote_comment(self, blogId: str, commentId: str, value: int = 1):
-        if value != 1: pass
-        elif value != -1: raise TypeError(f"Please fill the value by (-1 or 1) not ({value})")
+    def vote_comment(self, blogId: str, commentId: str, value: bool = True):
+        if value: value = 1
+        else: value = -1
+
         data = json.dumps({"value": value, "timestamp": int(timestamp() * 1000)})
         req = requests.post(api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1"), data=data, headers=headers.Headers(data=data).headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
@@ -367,7 +381,7 @@ class Local:
         data = json.dumps({"value": 4, "timestamp": int(timestamp() * 1000)})
 
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/vote?cv=1.2&value=4")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/vote?cv=1.2&value=4")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/vote?cv=1.2&value=4")
         else: raise TypeError("Please put wiki or blog Id")
 
         req = requests.post(url, headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data)
@@ -376,7 +390,7 @@ class Local:
 
     def unlike_blog(self, blogId: str = None, wikiId: str = None):
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/vote?eventSource=FeedList")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/vote?eventSource=FeedList")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/vote?eventSource=FeedList")
         else: raise TypeError("Please put wikiId or blogId")
 
         req = requests.delete(url, headers=self.headers, proxies=self.proxies)
@@ -394,8 +408,8 @@ class Local:
     def like_comment(self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None):
         data = json.dumps({"value": 1})
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
-        if userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
         req = requests.post(url, data=data, headers=headers)
@@ -404,8 +418,8 @@ class Local:
 
     def unlike_comment(self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None):
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/g-vote?eventSource=PostDetailView")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/g-vote?eventSource=PostDetailView")
-        if userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/g-vote?eventSource=PostDetailView")
+        elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
         req = requests.delete(url, headers=headers)
@@ -419,8 +433,8 @@ class Local:
         if isGuest: comType = "g-comment"
         else: comType = "comment"
         if userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/{comType}")
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/{comType}")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/{comType}")
+        elif blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/{comType}")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/{comType}")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
         data = json.dumps(data)
@@ -430,8 +444,8 @@ class Local:
 
     def delete_comment(self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None):
         if userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}")
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}")
+        elif blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}")
         else: raise TypeError(" ")
 
         req = requests.delete(url, headers=self.headers, proxies=self.proxies)
@@ -446,7 +460,7 @@ class Local:
 
     def get_blog_info(self, blogId: str = None, wikiId: str = None):
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}")
         else: raise TypeError("Please put a wiki or blog Id")
 
         req = requests.get(url, headers=self.headers, proxies=self.proxies)
@@ -468,12 +482,13 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return RecentBlogs(req.json()["blogList"]).RecentBlogs
 
-    def tip_coins(self, coins: int, chatId: str = None, blogId: str = None, wikiId: str = None, transactionId: str = str(UUID(hexlify(os.urandom(16)).decode("ascii")))):
+    def tip_coins(self, coins: int, chatId: str = None, blogId: str = None, wikiId: str = None, transactionId: str = None):
+        if transactionId is None: transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
         data = {"coins": int(coins), "tippingContext": {"transactionId": transactionId}, "timestamp": int(timestamp() * 1000)}
 
         if chatId: url = api(f"/x{self.comId}/s/chat/thread/{chatId}/tipping")
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/tipping")
-        if wikiId:
+        elif blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/tipping")
+        elif wikiId:
             url = api(f"/x{self.comId}/s/tipping")
             data["objectType"] = 2
             data["objectId"] = wikiId
@@ -497,10 +512,7 @@ class Local:
         else: return Json(req.json())
 
     def delete_message(self, chatId: str, messageId: str, asStaff: bool = False, reason: str = None):
-        data = {
-            "adminOpName": 102,
-            "timestamp": int(timestamp() * 1000)
-        }
+        data = {"adminOpName": 102, "timestamp": int(timestamp() * 1000)}
         if asStaff and reason: data["adminOpNote"] = {"content": reason}
         data = json.dumps(data)
         if asStaff: req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}/admin"), data=data, headers=headers.Headers(data=data).headers, proxies=self.proxies)
@@ -519,10 +531,7 @@ class Local:
             "uid": userId,
             "title": title,
             "content": reason,
-            "attachedObject": {
-                "objectId": userId,
-                "objectType": 0
-            },
+            "attachedObject": {"objectId": userId, "objectType": 0},
             "penaltyType": 1,
             "penaltyValue": time,
             "adminOpNote": {},
@@ -563,12 +572,12 @@ class Local:
             opN = 18
             opV = None
             url = api(f"/x{self.comId}/s/user-profile/{userId}/admin")
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/admin")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/admin")
-        if chatId: url = api(f"/x{self.comId}/s/chat/thread/{chatId}/admin")
+        elif blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/admin")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/admin")
+        elif chatId: url = api(f"/x{self.comId}/s/chat/thread/{chatId}/admin")
         else: raise TypeError("Please put a wiki or user or chat or blog Id")
 
-        data = {"adminOpNote": {"content": note},"adminOpName": opN,"adminOpValue": opV,"timestamp": int(timestamp() * 1000)}
+        data = {"adminOpNote": {"content": note}, "adminOpName": opN, "adminOpValue": opV, "timestamp": int(timestamp() * 1000)}
         req = requests.post(url, headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data)
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
@@ -579,9 +588,9 @@ class Local:
         if userId:
             opN = 19
             url = api(f"/x{self.comId}/s/user-profile/{userId}/admin")
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/admin")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/admin")
-        if chatId: url = api(f"/x{self.comId}/s/chat/thread/{chatId}/admin")
+        elif blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/admin")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/admin")
+        elif chatId: url = api(f"/x{self.comId}/s/chat/thread/{chatId}/admin")
         else: raise TypeError("Please put a wiki or user or chat or blog Id")
 
         data = {
@@ -669,7 +678,8 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def subscribe(self, userId: str, autoRenew: str = False, transactionId: str = str(UUID(hexlify(os.urandom(16)).decode("ascii")))):
+    def subscribe(self, userId: str, autoRenew: str = False, transactionId: str = None):
+        if transactionId is None: transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
         data = json.dumps({
             "paymentContext": {
                 "transactionId": transactionId,
@@ -703,7 +713,7 @@ class Local:
         if fansOnly: data["extensions"]["fansOnly"] = True
         if backgroundColor: data["extensions"] = {"backgroundColor": backgroundColor}
         if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}")
-        if wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}")
+        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}")
         else: raise TypeError("Please put blogId or wikiId")
 
         data = json.dumps(data)
@@ -739,14 +749,15 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return Json(req.json())
 
-    def kick(self, chatId: str, userId: str, allowRejoin: bool = True):
-        if not allowRejoin: allowRejoin = 1
-        if allowRejoin: allowRejoin = 0
-        req = requests.delete(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}?allowRejoin={allowRejoin}"), headers=self.headers, proxies=self.proxies)
+    def kick(self, chatId: str, userId: str, rejoin: bool = True):
+        if rejoin: rejoin = 1
+        if not rejoin: rejoin = 0
+
+        req = requests.delete(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}?allowRejoin={rejoin}"), headers=self.headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return Json(req.json())
 
-    def user_active_time(self):
+    def web_active_time(self):
         try:
             data = {"ndcId": self.comId}
             req = requests.post("https://aminoapps.com/api/community/stats/web-user-active-time", json=data, headers=self.web_headers)
@@ -757,7 +768,12 @@ class Local:
             return Json(req.json())
         except: raise Exception("Unknown error")
 
-    def send_active_obj(self, startTime: int = None, endTime: int = None, optInAdsFlags: int = 2147483647, tz: int = -timezone // 1000, timers: list = None, timestamp: int = int(timestamp() * 1000)):
+    def block(self, userId: str):
+        req = requests.post(api(f"/x{self.comId}/s/block/{userId}"), headers=self.headers, proxies=self.proxies)
+        if "OK" not in req.json()["api:message"]: return CheckExceptions(req.json())
+        return Json(req.json())
+
+    def send_active_time(self, startTime: int = None, endTime: int = None, optInAdsFlags: int = 2147483647, tz: int = -timezone // 1000, timers: list = None, timestamp: int = int(timestamp() * 1000)):
         data = {
             "userActiveTimeChunkList": [{"start": startTime, "end": endTime}],
             "timestamp": timestamp,
@@ -771,3 +787,18 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return Json(req.json())
 
+    def transfer_host(self, chatId: str, userIds: list):
+        data = json.dumps({
+            "uidList": userIds,
+            "timestamp": int(timestamp() * 1000)
+        })
+
+        req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer"), headers=headers.Headers(data=data).headers, data=data, proxies=self.proxies)
+        if req.status_code != 200: return CheckExceptions(req.json())
+        else: return Json(req.json())
+
+    def accept_host(self, chatId: str, requestId: str):
+        data = json.dumps({})
+        req = requests.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer/{requestId}/accept"), headers=headers.Headers(data=data).headers, data=data, proxies=self.proxies)
+        if req.status_code != 200: return CheckExceptions(req.json())
+        else: return Json(req.json())
